﻿using System;
using ETHZ.C0284H.Calculator.Instruct.Visitor;
using ETHZ.C0284H.Calculator.UI;

namespace ETHZ.C0284H.Calculator
{
    /// <summary>
    ///     Calculator controller.
    /// </summary>
    internal class CalculatorController
    {
        private readonly ReversePolishNotationCalculator _calculator;

        private readonly CalculatorExecutor _executor;
        private readonly ICalculatorView _view;

        /// <summary>
        ///     Constructor.
        /// </summary>
        /// <param name="calculator"></param>
        /// <param name="view"></param>
        public CalculatorController(ReversePolishNotationCalculator calculator, ICalculatorView view)
        {
            if (calculator == null)
            {
                throw new ArgumentNullException("calculator");
            }
            if (view == null)
            {
                throw new ArgumentNullException("view");
            }

            _calculator = calculator;
            _view = view;
            _executor = new CalculatorExecutor(_calculator);
        }

        public void StartViewNotification()
        {
            // Subscribe the controller to the new instruct event from the view.
            _view.InstructEvent += OnNewInstruct; 
        }

        /// <summary>
        ///     Handler for new instruct event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        public void OnNewInstruct(object sender, InstructEventArgs args)
        {
            args.Instruct.Accept(_executor);
        }
    }
}