
using System;
using ETHZ.C0284H.Calculator.CalculatorNode.Visitor;

namespace ETHZ.C0284H.Calculator.CalculatorNode
{

	/// <summary>
	/// Operator class
	/// </summary>
	public abstract class AbstractOperator: INode
	{

		private readonly string _stringRep; 
		/// <summary>
		/// Constructs a new instance of operator with specified string representation.
		/// </summary>
		/// <param name="representation">
		/// A string representation of operator. Can't be null.
		/// </param>
		protected AbstractOperator(string representation)
		{
		    if (representation == null)
		    {
		        throw  new ArgumentNullException("representation");
		    }
			_stringRep = representation;
		}
		
		/// <summary>
		/// Descendants will implement double dispatch using themselves as specific operand and the specific visitor passed as argument
		/// </summary>
		/// <param name="visitor">
		/// A <see cref="IVisitor"/>
		/// </param>
		public abstract void Accept(IVisitor visitor);
		
		/// <summary>
		/// String representation of an operator
		/// </summary>
		/// <returns>
		/// A <see cref="System.String"/>
		/// </returns>
		public override string ToString ()
		{
			return string.Format("{0}", _stringRep);
		}                  
	}
}
